package me.lensfrex.manager.data.manager;

import me.lensfrex.manager.data.Student;

import java.util.ArrayList;
import java.util.HashMap;

/**
 * 对Student类的操作的上层封装，有较为多的操作
 */
public class StudentManager {
    private final ArrayList<Student> students = new ArrayList<>();
    private final ManagerTool managerTool = new ManagerTool(students);

    /**
     * 增添学生。输入的字符串格式应为 [id] [name] ...[subjectName,score]
     * 如： 23333333 Lag English,100 Math,99 History,98
     *
     * @param input 待添加的学生信息
     */
    public void addStudent(String input) {
        String[] split = input.split(" ");

        long id = Long.parseLong(split[0]);
        String name = split[1];

        HashMap<String, Integer> score = new HashMap<>();
        String[] scoreInput;
        for (int i = 2; i < split.length; i++) {
            scoreInput = split[i].split(",");
            score.put(scoreInput[0], Integer.parseInt(scoreInput[1]));
        }

        managerTool.addStudent(id, name, score);
    }


    // 要是Java能像go语言那样返回两个返回值就好了...
    // 这样就能同时返回平均值和总值了...
    // 做宝可梦.jpg
    // 还是老老实实写两方法吧...

    /**
     * 获取班级某科目平均成绩
     *
     * @param subject 欲获取的科目
     * @return 科目平均成绩
     */
    public float getSubjectAvgScore(String subject) {
        return (float) getSubjectScore(subject) / managerTool.getStudentAmount();
    }

    /**
     * 获取班级某科目总成绩
     *
     * @param subject 欲获取的科目
     * @return 科目总成绩
     */
    public int getSubjectScore(String subject) {
        ArrayList<Integer> scores = new ArrayList<>(managerTool.getSubjectScores(subject).values());
        int sum = 0;
        for (int score : scores) {
            sum += score;
        }
        return sum;
    }

    /**
     * 获取每门科目的总成绩
     *
     * @return 包含每门课总成绩的HashMap，key为科目名称，value为该科目的总成绩
     */
    public HashMap<String, Integer> getAllSubjectScore() {
        ArrayList<String> subjects = managerTool.getAllSubjects();
        HashMap<String, Integer> scores = new HashMap<>();

        for (String subject : subjects) {
            scores.put(subject, getSubjectScore(subject));
        }

        return scores;
    }

    /**
     * 获取每门科目的平均成绩
     *
     * @return 包含每门课总成绩的HashMap，key为科目名称，value为该科目的平均成绩
     */
    public HashMap<String, Float> getAllSubjectAvgScore() {
        ArrayList<String> subjects = managerTool.getAllSubjects();
        HashMap<String, Float> scores = new HashMap<>();

        for (String subject : subjects) {
            scores.put(subject, getSubjectAvgScore(subject));
        }

        return scores;
    }

    /**
     *  获取所有科目
     * @return 包含有科目名称的ArrayList
     */
    public ArrayList<String> getSubjects() {
        return managerTool.getAllSubjects();
    }

    /**
     * 按学生总分由高到低生成名次表
     *
     * @return 名次表字符串
     */
    public String generateRankingTable() {
        StringBuilder stringBuilder = new StringBuilder("Ranking  |  Name  |  General Score\n");
        ArrayList<Student> sortStudentList = managerTool.getSortedStudentList(ManagerTool.SORT_COMPARE_BY_GENERAL_SCORE);

        for (int i = 0; i < sortStudentList.size(); i++) {
            stringBuilder
                    .append(i + 1).append("  |  ")
                    .append(sortStudentList.get(i).getName()).append("  |  ")
                    .append(students.get(i).getGeneralScore())
                    .append('\n');
        }

        return stringBuilder.toString();
    }

    /**
     * 按学生学号由小到大生成成绩表
     *
     * @return 成绩表字符串
     */
    public String generateScoreTableById() {
        StringBuilder stringBuilder = new StringBuilder("ID  |  Name  |  General Score  |  Subject1: score Subject2: score ...\n");
        ArrayList<Student> sortStudentList = managerTool.getSortedStudentList(ManagerTool.SORT_COMPARE_BY_ID);

        for (Student student : sortStudentList) {
            stringBuilder
                    .append(student.getId()).append("  |  ")
                    .append(student.getName()).append("  |  ")
                    .append(student.getGeneralScore()).append("  |  ")
                    .append(listScore(student)).append('\n');
        }

        return stringBuilder.toString();
    }

    /**
     * 按学生学号由小到大生成成绩表
     *
     * @return 成绩表字符串
     */
    public String generateScoreTableByName() {
        StringBuilder stringBuilder = new StringBuilder("Name  |  Id  |  General Score  |  Subject1: score Subject2: score ...\n");
        ArrayList<Student> sortStudentList = managerTool.getSortedStudentList(ManagerTool.SORT_COMPARE_BY_NAME);

        for (Student student : sortStudentList) {
            stringBuilder
                    .append(student.getName()).append("  |  ")
                    .append(student.getId()).append("  |  ")
                    .append(student.getGeneralScore()).append("  |  ")
                    .append(listScore(student)).append('\n');
        }

        return stringBuilder.toString();
    }

    /**
     * 列出某一学生的成绩
     *
     * @param student 学生
     * @return 成绩字符串
     */
    private String listScore(Student student) {
        StringBuilder stringBuilder = new StringBuilder();
        ArrayList<String> subjects = managerTool.getAllSubjects();

        for (String subject : subjects) {
            stringBuilder
                    .append(subject).append(": ")
                    .append(student.getScore().get(subject)).append(' ');
        }
        return stringBuilder.toString();
    }

}
